﻿--------------------------------------------------------------------------------
--  iTip
--
--  Description: Adds useful information to item tooltips.
--  Author: CHiLLZ <chillz@gmail.com>
--  License: Copyright 2006-2016, all rights reserved.
--  Website: http://www.curse.com/addons/wow/itip
--
--------------------------------------------------------------------------------

ITIP = {};
ITIP.version = 1.7;
ITIP.minimum_version = 1.6;
ITIP.debug = false;
ITIP.loaded = false;


--------------------------------------------------------------------------------
--  HELPER FUNCTIONS
--------------------------------------------------------------------------------

ITIP.Toggle = function( value )
	local new_val;
	
	if value then
		new_val = false;
	else
		new_val = true;
	end
	
	return new_val;
end

ITIP.BoolToString = function( value, color, trueColor, falseColor )
	local readable,c1,c2;
	
	if trueColor then c1 = "|c"..trueColor; else c1 = "|cFF00FF00"; end
	if falseColor then c2 = "|c"..falseColor; else c2 = "|cFFFF0000"; end
	
	if (value == nil) then
		readable = nil;
	elseif value then
		if color then
			readable = {c1.."TRUE|r",c1.."ON|r",c1.."ENABLED|r"};
		else
			readable = {"TRUE","ON","ENABLED"};
		end
	else
		if color then
			readable = {c2.."FALSE|r",c2.."OFF|r",c2.."DISABLED|r"};
		else
			readable = {"FALSE","OFF","DISABLED"};
		end
	end
	
	return readable;
end

ITIP.Message = function( body, color )
	local prefix = "|cFF33CCFF[iTip]|r  ";
	local msg;
	
	if color then
		msg = prefix.."|c"..color..body;
	else
		msg = prefix..body;
	end
	
	DEFAULT_CHAT_FRAME:AddMessage(msg);
end

--------------------------------------------------------------------------------
--  BOOTSTRAP
--------------------------------------------------------------------------------

ITIP.Load = function( self )
	self:RegisterEvent("ADDON_LOADED");
	-- register commandline interface
	SLASH_iTip1 = "/itip";
	SlashCmdList["iTip"] = ITIP.Command;
end

ITIP.Event = function( self, event, ... )
	local arg = ...;
	
	if (event == "ADDON_LOADED") and (arg == "iTip") then
		self:UnregisterEvent("ADDON_LOADED");
		ITIP.Init();
		ITIP.loaded = ITIP.Startup();
	end
end

--------------------------------------------------------------------------------
--  CORE FUNCTIONS
--------------------------------------------------------------------------------

ITIP.Init = function( reset )
	-- default configuration values
	ITIP.Defaults = {};
	ITIP.Defaults.version = ITIP.version;
	ITIP.Defaults.enabled = true;
	ITIP.Defaults.equipment = false;
	
	if reset or (ITIP_CONFIG == nil) or (ITIP_CONFIG.version < ITIP.minimum_version) then
		-- restore defaults
		ITIP_CONFIG = ITIP.Defaults;
		if ITIP.debug then ITIP.Message("ITIP.Init() Default configuration loaded."); end
	end
end

ITIP.Startup = function()
	if ITIP.debug then ITIP.Message("ITIP.Startup() running..."); end
	local result;
	
	-- if config is still missing at this point, something is very wrong
	if (ITIP_CONFIG == nil) then
		ITIP.Message("|cFFFF0000 ERROR LOADING SAVED VARIABLES!|r\nThis can happen if your interface takes too long loading. Try waiting a few minutes and then reload your interface by typing:  /reload");
		-- startup failed!
		result = false;
	else
		-- localization
		local lang = GetLocale();
		-- validate locale
		if (ITIP_TRANSLATIONS[lang] == nil) or (#ITIP_TRANSLATIONS[lang] < 3) then
			ITIP.labels = ITIP.TRANSLATIONS["enUS"];
		else
			ITIP.labels = ITIP_TRANSLATIONS[lang];
		end
		
		------------------------------------------------------------
		--  REGISTER HOOK
		------------------------------------------------------------
		GameTooltip:HookScript("OnTooltipSetItem", ITIP.Hook);
		
		-- output
		local status;
		local val = ITIP.BoolToString(ITIP_CONFIG.enabled,true);
		
		if (ITIP.version > ITIP_CONFIG.version) then
			ITIP_CONFIG.version = ITIP.version;
			status = val[3]..". Updated to v"..tostring(ITIP.version).."!";
		else
			status = "v"..tostring(ITIP.version).." "..val[3]..".";
		end
		local msg = status.." For help, type:  /itip";
		
		ITIP.Message(msg);
		
		-- startup complete
		result = true;
	end
	
	return result;
end

ITIP.Hook = function( tooltip )
	-- debugging output
	if ITIP.debug then ITIP.Message("Hook function called for "..tooltip:GetName().." "..tooltip:GetItem()); end
	
	if ITIP_CONFIG.enabled then
		local itemName, itemLink = tooltip:GetItem();
		
		-- make sure this is actually an item
		if itemLink then
			-- extract the item ID from the link
			local prefix, itemID = strsplit(':', itemLink);
			
			-- get all the rest of the item info
			local sName, sLink, iQuality, iLevel, iMinLevel, sType, sSubType, iStackSize, sEquipLoc = GetItemInfo(itemID);
			
			------------------------------------------------------------
			--  INSERT TOOLTIP LINES
			------------------------------------------------------------
			if sType then
				-- ITEM CATEGORY
				if (strlen(sEquipLoc) == 0) or (ITIP_CONFIG.equipment == true) then 
					tooltip:AddLine("\n"..sType.." ("..sSubType..")");
				end
				
				-- STACK SIZE
				if (iStackSize > 1) then tooltip:AddLine(ITIP.labels[2]..":  "..iStackSize, 1,1,1); end
				
				-- ITEM LEVEL
				if (strlen(sEquipLoc) == 0) then tooltip:AddLine(ITIP.labels[1]..":  "..iLevel, 1,1,1); end
				
				-- ITEM ID
				tooltip:AddLine(ITIP.labels[3]..":  "..itemID, 1,1,1);
			
			elseif ITIP.debug then
				-- the item query failed
				ITIP.Message("Error: Bad result from GetItemInfo()");
			end
		end
	end
end

--------------------------------------------------------------------------------
--  COMMAND INTERFACE
--------------------------------------------------------------------------------

ITIP.Command = function( input, editbox )
	-- parse input
	local cmd, arg, val = strsplit(' ', input);
	cmd = strlower(cmd);
	
	if ITIP.loaded then
		-- actions
		if (string.len(cmd) == 0) or (cmd == "help") then
			ITIP.Help();
		elseif (cmd == "reset") then
			ITIP.Reset();
		elseif (cmd == "show") then
			ITIP.Show();
		elseif (cmd == "equip") then
			ITIP.Filter("equipment");
		else
			ITIP.Help(true);
		end
	else
		ITIP.Message("There was a problem loading your interface. Please try reloading your UI by typing:  /reload");
	end
end

ITIP.Help = function( err )
	local output;
	
	local val = {}
	val.enabled = ITIP.BoolToString(ITIP_CONFIG.enabled,true);
	val.equipment = ITIP.BoolToString(ITIP_CONFIG.equipment,true);
	
	if err then
		output = "Unknown command. For more info type:  /itip help";
	else
		output = "Available commands:\n"..
		"|cFFFFFF00/itip help |r\n"..
		"    Shows this help info.\n"..
		"|cFFFFFF00/itip reset |r\n"..
		"    Restore default configuration settings.\n    THIS WILL RELOAD YOUR INTERFACE!\n"..
		"|cFFFFFF00/itip show |r\n"..
		"    Turns the entire addon on/off. Currently: "..val.enabled[2].."\n"..
		"|cFFFFFF00/itip equip |r\n"..
		"    Toggle showing the item category for equipment. Currently: "..val.equipment[2];
	end
	
	ITIP.Message(output);
end

ITIP.Reset = function()
	-- restore default config
	ITIP.Init(true);
	-- save the changes
	ReloadUI();
end

ITIP.Show = function( input, editbox )
	-- change config setting
	ITIP_CONFIG.enabled = ITIP.Toggle(ITIP_CONFIG.enabled);
	
	-- output
	local val = ITIP.BoolToString(ITIP_CONFIG.enabled,true);
	local msg = "Tooltip enhancement is now "..val[2].."."
	ITIP.Message(msg);
end

ITIP.Filter = function( criteria )
	if (criteria == "equipment") then
		-- change config setting
		ITIP_CONFIG.equipment = ITIP.Toggle(ITIP_CONFIG.equipment);
		
		-- output
		local label = "|cFFFFFF00equipment|r";
		local val = ITIP.BoolToString(ITIP_CONFIG.equipment,true);
		local msg = "Showing the item category of "..label.." is now "..val[2]..".";
		
		ITIP.Message(msg);
	end
end
